import React, { lazy, Suspense } from 'react';
import { createRoot, hydrateRoot } from 'react-dom/client';
import { BrowserRouter } from 'react-router-dom';
import './styles/critical.css'; // Import only critical styles initially

// Minimal loading indicator with inline styles for fast rendering
const LoadingIndicator = () => (
  <div className="loading-indicator">
    <div className="spinner"></div>
  </div>
);

// Determine if the device is mobile
const isMobile = 
  typeof window !== 'undefined' && 
  (window.innerWidth <= 640 || 
   navigator.userAgent.includes('Mobile') || 
   navigator.userAgent.includes('Android'));

// Dynamic import of App with minimal bundle
const App = lazy(() => import('./App.tsx').then(module => {
  if (window.performance && window.performance.mark) {
    window.performance.mark('app-component-loaded');
  }
  return module;
}));

// Create React root with efficient rendering approach
const rootElement = document.getElementById("root");

// Function to optimize rendering
const renderApp = () => {
  if (!rootElement) return;
  
  // Remove loading elements
  const spinner = document.querySelector('.loading-spinner');
  const placeholders = document.getElementById('content-placeholders');
  
  if (spinner) {
    if (spinner.parentNode) {
      spinner.parentNode.removeChild(spinner);
    }
  }
  
  if (placeholders) {
    placeholders.style.display = 'none';
  }
  
  const app = (
    <BrowserRouter>
      {process.env.NODE_ENV === 'development' ? (
        <React.StrictMode>
          <Suspense fallback={<LoadingIndicator />}>
            <App />
          </Suspense>
        </React.StrictMode>
      ) : (
        <Suspense fallback={<LoadingIndicator />}>
          <App />
        </Suspense>
      )}
    </BrowserRouter>
  );

  // Use hydration for SSR in production
  if (import.meta.env.PROD) {
    hydrateRoot(rootElement, app);
  } else {
    createRoot(rootElement).render(app);
  }
  
  // Initialize performance optimizations in idle callback
  if ('requestIdleCallback' in window) {
    window.requestIdleCallback(() => {
      // Initialize only essential performance optimizations on mobile
      import('./utils/performance').then(module => {
        if (module.initializePerformanceOptimizations) {
          module.initializePerformanceOptimizations();
        }
      });
      
      // Only load additional resources on desktop or after interaction
      if (!isMobile) {
        loadDesktopResources();
      }
    }, { timeout: isMobile ? 3000 : 2000 });
  } else {
    setTimeout(() => {
      import('./utils/performance').then(module => {
        if (module.initializePerformanceOptimizations) {
          module.initializePerformanceOptimizations();
        }
      });
      
      if (!isMobile) {
        loadDesktopResources();
      }
    }, isMobile ? 500 : 300);
  }
};

// Function to load desktop-only resources
const loadDesktopResources = () => {
  // These resources are only needed for desktop
  import('./styles/animations.css');
  
  // Load desktop-specific utilities in idle time
  if ('requestIdleCallback' in window) {
    window.requestIdleCallback(() => {
      import('./utils/webVitals');
    }, { timeout: 4000 });
  }
};

// Load mobile interactive resources after user interaction
const loadAfterInteraction = () => {
  if (isMobile) {
    // Add event listener for first interaction
    const interactionEvents = ['click', 'touchstart'];
    
    const loadInteractiveResources = () => {
      // Remove all listeners after first interaction
      interactionEvents.forEach(event => {
        document.removeEventListener(event, loadInteractiveResources);
      });
      
      // Now load the rest of the resources
      import('./styles/animations.css');
      
      // Set a flag to remember that resources are loaded
      localStorage.setItem('resourcesLoaded', 'true');
    };
    
    // Check if resources were previously loaded in this session
    if (localStorage.getItem('resourcesLoaded') === 'true') {
      loadInteractiveResources();
    } else {
      interactionEvents.forEach(event => {
        document.addEventListener(event, loadInteractiveResources, { 
          once: true, 
          passive: true 
        });
      });
    }
  }
};

// Optimize initial hydration
const startHydration = () => {
  if (document.readyState !== 'loading') {
    requestAnimationFrame(renderApp);
  } else {
    document.addEventListener('DOMContentLoaded', () => {
      requestAnimationFrame(renderApp);
    });
  }
  loadAfterInteraction();
};

// Start hydration process
startHydration();

// Handle mobile-specific setup
if (typeof window !== 'undefined') {
  // Set viewport height for mobile browsers
  const setVh = () => {
    const vh = window.innerHeight * 0.01;
    document.documentElement.style.setProperty('--vh', `${vh}px`);
  };
  
  window.addEventListener('resize', setVh, { passive: true });
  window.addEventListener('orientationchange', setVh, { passive: true });
  setVh();
  
  // Mark first paint timing
  if (window.performance && window.performance.mark) {
    window.performance.mark('app-init');
    
    // Record FCP for analytics
    if ('PerformanceObserver' in window) {
      const perfObserver = new PerformanceObserver((entries) => {
        const fcp = entries.getEntriesByName('first-contentful-paint')[0];
        if (fcp) {
          console.info(`FCP: ${Math.round(fcp.startTime)}ms`);
          window.performance.mark('fcp-recorded');
        }
      });
      
      perfObserver.observe({ type: 'paint', buffered: true });
    }
  }
}
